#include "at91sam7s64.h"
#include "uart_d.h"


#define BAUD_RATE				115200		// Baud Rate
#define MCK						48054857	// MCK (PLLRC div by 2)


// main_clock:	peripheral clock
// baud_rate:	UART baudrate
unsigned int BaudrateCalc(const unsigned int main_clock, const unsigned int baud_rate)
{
	unsigned int	baud_value = (10 * main_clock/(baud_rate * 16));

	if((baud_value % 10) >= 5)
		baud_value = (baud_value / 10) + 1;
	else
		baud_value /= 10;
	return baud_value;
}


void DEBUGU_Init(void)
{
	// DEBUGU clock enable
	*AT91C_PMC_PCER = 1 << AT91C_ID_SYS;

	*AT91C_PIOA_PDR  = AT91C_PA10_DTXD | AT91C_PA9_DRXD;   //Disable PIO
	*AT91C_PIOA_ASR  = AT91C_PA10_DTXD | AT91C_PA9_DRXD;   //Enable PeripherialA

	*AT91C_DBGU_CR	 = AT91C_US_RSTRX | AT91C_US_RSTTX;
	*AT91C_DBGU_BRGR = BaudrateCalc(MCK, BAUD_RATE);
	*AT91C_DBGU_MR   = AT91C_US_PAR_NONE | AT91C_US_CHMODE_NORMAL;
	*AT91C_DBGU_CR   = AT91C_US_RXEN | AT91C_US_TXEN;
}


int DEBUGU_IsRxReady(void)
{
	if(*AT91C_DBGU_CSR & AT91C_US_RXRDY)
		return 1;
	else
		return 0;
}


unsigned int DEBUGU_Read(char *pData, unsigned int length)
{
	unsigned int size = length;

	while(size--)
	{
		while(!(*AT91C_DBGU_CSR & AT91C_US_RXRDY));
		*pData++ = (*AT91C_DBGU_RHR);
	}

	return length;
}


unsigned int DEBUGU_Write(char *pData, unsigned int length)
{
	unsigned int size = length;

	while(size--)
	{
		while((*AT91C_DBGU_CSR & AT91C_US_TXRDY) == 0);
		*AT91C_DBGU_THR = *pData++;
	}

	return length;
}


void DEBUGU_WriteStr(char *pData)
{
	while(*pData)
	{
		if(*pData == '\n')
		{
			while((*AT91C_DBGU_CSR & AT91C_US_TXRDY) == 0);
			*AT91C_DBGU_THR = '\r';
		}

		while((*AT91C_DBGU_CSR & AT91C_US_TXRDY) == 0);
		*AT91C_DBGU_THR = *pData++;
	}
}
